@@ -217,6 +217,9 @@ fn default_field_name(dt: &DataType) -> &str {
         DataType::Union(_, _, _) => "union",
         DataType::Dictionary(_, _) => "map",
         DataType::Map(_, _) => unimplemented!("Map support not implemented"),
+        DataType::RunEndEncoded(_, _) => {
+            unimplemented!("RunEndEncoded support not implemented")
+        }
         DataType::Decimal128(_, _) => "decimal",
         DataType::Decimal256(_, _) => "decimal",
     }
@@ -424,9 +427,9 @@ mod test {
               } ]
             }"#,
         );
-        assert!(schema.is_ok(), "{:?}", schema);
+        assert!(schema.is_ok(), "{schema:?}");
         let arrow_schema = to_arrow_schema(&schema.unwrap());
-        assert!(arrow_schema.is_ok(), "{:?}", arrow_schema);
+        assert!(arrow_schema.is_ok(), "{arrow_schema:?}");
         let expected = Schema::new(vec![
             Field::new("id", Int32, true),
             Field::new("bool_col", Boolean, true),
@@ -446,7 +449,7 @@ mod test {
     #[test]
     fn test_non_record_schema() {
         let arrow_schema = to_arrow_schema(&AvroSchema::String);
-        assert!(arrow_schema.is_ok(), "{:?}", arrow_schema);
+        assert!(arrow_schema.is_ok(), "{arrow_schema:?}");
         assert_eq!(
             arrow_schema.unwrap(),
             Schema::new(vec![Field::new("", Utf8, false)])
