@@ -20,11 +20,18 @@
 
 pub mod aggregate_statistics;
 pub mod coalesce_batches;
-pub mod enforcement;
+pub mod dist_enforcement;
+pub mod global_sort_selection;
 pub mod join_selection;
 pub mod optimizer;
+pub mod pipeline_checker;
 pub mod pruning;
 pub mod repartition;
+pub mod sort_enforcement;
 mod utils;
 
+pub mod pipeline_fixer;
+#[cfg(test)]
+pub mod test_utils;
+
 pub use optimizer::PhysicalOptimizerRule;
