@@ -25,13 +25,17 @@
 //!
 //! A non-realistic but easy to follow example for sort removals: Assume that we
 //! somehow get the fragment
-//! "SortExec: [nullable_col@0 ASC]",
-//! "  SortExec: [non_nullable_col@1 ASC]",
+//!
+//! ```text
+//! SortExec: expr=[nullable_col@0 ASC]
+//!   SortExec: expr=[non_nullable_col@1 ASC]
+//! ```
+//!
 //! in the physical plan. The first sort is unnecessary since its result is overwritten
 //! by another SortExec. Therefore, this rule removes it from the physical plan.
 use crate::config::ConfigOptions;
 use crate::error::Result;
-use crate::physical_optimizer::utils::add_sort_above_child;
+use crate::physical_optimizer::utils::add_sort_above;
 use crate::physical_optimizer::PhysicalOptimizerRule;
 use crate::physical_plan::coalesce_partitions::CoalescePartitionsExec;
 use crate::physical_plan::limit::{GlobalLimitExec, LocalLimitExec};
@@ -49,7 +53,7 @@ use itertools::{concat, izip};
 use std::iter::zip;
 use std::sync::Arc;
 
-/// This rule inspects SortExec's in the given physical plan and removes the
+/// This rule inspects `SortExec`'s in the given physical plan and removes the
 /// ones it can prove unnecessary.
 #[derive(Default)]
 pub struct EnforceSorting {}
@@ -65,15 +69,28 @@ impl EnforceSorting {
 /// leading to `SortExec`s.
 #[derive(Debug, Clone)]
 struct ExecTree {
+    /// The `ExecutionPlan` associated with this node
+    pub plan: Arc<dyn ExecutionPlan>,
     /// Child index of the plan in its parent
     pub idx: usize,
     /// Children of the plan that would need updating if we remove leaf executors
     pub children: Vec<ExecTree>,
-    /// The `ExecutionPlan` associated with this node
-    pub plan: Arc<dyn ExecutionPlan>,
 }
 
 impl ExecTree {
+    /// Create new Exec tree
+    pub fn new(
+        plan: Arc<dyn ExecutionPlan>,
+        idx: usize,
+        children: Vec<ExecTree>,
+    ) -> Self {
+        ExecTree {
+            plan,
+            idx,
+            children,
+        }
+    }
+
     /// This function returns the executors at the leaves of the tree.
     fn get_leaves(&self) -> Vec<Arc<dyn ExecutionPlan>> {
         if self.children.is_empty() {
@@ -92,7 +109,7 @@ struct PlanWithCorrespondingSort {
     // For every child, keep a subtree of `ExecutionPlan`s starting from the
     // child until the `SortExec`(s) -- could be multiple for n-ary plans like
     // Union -- that determine the output ordering of the child. If the child
-    // has no connection to any sort, simpliy store None (and not a subtree).
+    // has no connection to any sort, simply store None (and not a subtree).
     sort_onwards: Vec<Option<ExecTree>>,
 }
 
@@ -123,11 +140,7 @@ impl PlanWithCorrespondingSort {
                 // that maintain this ordering. If we just saw a order imposing
                 // operator, we reset the tree and start accumulating.
                 if is_sort(plan) {
-                    return Some(ExecTree {
-                        idx,
-                        plan: item.plan,
-                        children: vec![],
-                    });
+                    return Some(ExecTree::new(item.plan, idx, vec![]));
                 } else if is_limit(plan) {
                     // There is no sort linkage for this path, it starts at a limit.
                     return None;
@@ -155,11 +168,7 @@ impl PlanWithCorrespondingSort {
                 if !children.is_empty() {
                     // Add parent node to the tree if there is at least one
                     // child with a subtree:
-                    Some(ExecTree {
-                        idx,
-                        plan: item.plan,
-                        children,
-                    })
+                    Some(ExecTree::new(item.plan, idx, children))
                 } else {
                     // There is no sort linkage for this child, do nothing.
                     None
@@ -238,11 +247,7 @@ impl PlanWithCorrespondingCoalescePartitions {
                 // operator, we reset the tree and start accumulating.
                 let plan = item.plan;
                 if plan.as_any().is::<CoalescePartitionsExec>() {
-                    Some(ExecTree {
-                        idx,
-                        plan,
-                        children: vec![],
-                    })
+                    Some(ExecTree::new(plan, idx, vec![]))
                 } else if plan.children().is_empty() {
                     // Plan has no children, there is nothing to propagate.
                     None
@@ -263,11 +268,7 @@ impl PlanWithCorrespondingCoalescePartitions {
                     if children.is_empty() {
                         None
                     } else {
-                        Some(ExecTree {
-                            idx,
-                            plan,
-                            children,
-                        })
+                        Some(ExecTree::new(plan, idx, children))
                     }
                 }
             })
@@ -342,12 +343,12 @@ impl PhysicalOptimizerRule for EnforceSorting {
 }
 
 /// This function turns plans of the form
-///      "SortExec: [a@0 ASC]",
+///      "SortExec: expr=[a@0 ASC]",
 ///      "  CoalescePartitionsExec",
 ///      "    RepartitionExec: partitioning=RoundRobinBatch(8), input_partitions=1",
 /// to
 ///      "SortPreservingMergeExec: [a@0 ASC]",
-///      "  SortExec: [a@0 ASC]",
+///      "  SortExec: expr=[a@0 ASC]",
 ///      "    RepartitionExec: partitioning=RoundRobinBatch(8), input_partitions=1",
 /// by following connections from `CoalescePartitionsExec`s to `SortExec`s.
 /// By performing sorting in parallel, we can increase performance in some scenarios.
@@ -361,19 +362,21 @@ fn parallelize_sorts(
     let mut coalesce_onwards = requirements.coalesce_onwards;
     // We know that `plan` has children, so `coalesce_onwards` is non-empty.
     if coalesce_onwards[0].is_some() {
-        if let Some(sort_exec) = plan.as_any().downcast_ref::<SortExec>() {
+        if (is_sort(&plan) || is_sort_preserving_merge(&plan))
+            // Make sure that Sort is actually global sort
+            && plan.output_partitioning().partition_count() == 1
+        {
             // If there is a connection between a `CoalescePartitionsExec` and a
-            // `SortExec` that satisfy the requirements (i.e. they don't require a
-            // single partition), then we can replace the `CoalescePartitionsExec`
-            // + `SortExec` cascade with a `SortExec` + `SortPreservingMergeExec`
+            // Global Sort that satisfy the requirements (i.e. intermediate
+            // executors  don't require single partition), then we can
+            // replace the `CoalescePartitionsExec`+ GlobalSort cascade with
+            // the `SortExec` + `SortPreservingMergeExec`
             // cascade to parallelize sorting.
             let mut prev_layer = plan.clone();
-            update_child_to_change_coalesce(
-                &mut prev_layer,
-                &mut coalesce_onwards[0],
-                Some(sort_exec),
-            )?;
-            let spm = SortPreservingMergeExec::new(sort_exec.expr().to_vec(), prev_layer);
+            update_child_to_remove_coalesce(&mut prev_layer, &mut coalesce_onwards[0])?;
+            let sort_exprs = get_sort_exprs(&plan)?;
+            add_sort_above(&mut prev_layer, sort_exprs.to_vec())?;
+            let spm = SortPreservingMergeExec::new(sort_exprs.to_vec(), prev_layer);
             return Ok(Some(PlanWithCorrespondingCoalescePartitions {
                 plan: Arc::new(spm),
                 coalesce_onwards: vec![None],
@@ -381,11 +384,7 @@ fn parallelize_sorts(
         } else if plan.as_any().is::<CoalescePartitionsExec>() {
             // There is an unnecessary `CoalescePartitionExec` in the plan.
             let mut prev_layer = plan.clone();
-            update_child_to_change_coalesce(
-                &mut prev_layer,
-                &mut coalesce_onwards[0],
-                None,
-            )?;
+            update_child_to_remove_coalesce(&mut prev_layer, &mut coalesce_onwards[0])?;
             let new_plan = plan.with_new_children(vec![prev_layer])?;
             return Ok(Some(PlanWithCorrespondingCoalescePartitions {
                 plan: new_plan,
@@ -449,12 +448,8 @@ fn ensure_sorting(
                     // Make sure we preserve the ordering requirements:
                     update_child_to_remove_unnecessary_sort(child, sort_onwards, &plan)?;
                     let sort_expr = required_ordering.to_vec();
-                    *child = add_sort_above_child(child, sort_expr)?;
-                    *sort_onwards = Some(ExecTree {
-                        idx,
-                        plan: child.clone(),
-                        children: vec![],
-                    })
+                    add_sort_above(child, sort_expr)?;
+                    *sort_onwards = Some(ExecTree::new(child.clone(), idx, vec![]));
                 }
                 if let Some(tree) = sort_onwards {
                     // For window expressions, we can remove some sorts when we can
@@ -470,12 +465,8 @@ fn ensure_sorting(
             }
             (Some(required), None) => {
                 // Ordering requirement is not met, we should add a `SortExec` to the plan.
-                *child = add_sort_above_child(child, required.to_vec())?;
-                *sort_onwards = Some(ExecTree {
-                    idx,
-                    plan: child.clone(),
-                    children: vec![],
-                })
+                add_sort_above(child, required.to_vec())?;
+                *sort_onwards = Some(ExecTree::new(child.clone(), idx, vec![]));
             }
             (None, Some(_)) => {
                 // We have a `SortExec` whose effect may be neutralized by
@@ -528,11 +519,11 @@ fn analyze_immediate_sort_removal(
                             sort_exec.expr().to_vec(),
                             sort_input,
                         ));
-                    let new_tree = ExecTree {
-                        idx: 0,
-                        plan: new_plan.clone(),
-                        children: sort_onwards.iter().flat_map(|e| e.clone()).collect(),
-                    };
+                    let new_tree = ExecTree::new(
+                        new_plan.clone(),
+                        0,
+                        sort_onwards.iter().flat_map(|e| e.clone()).collect(),
+                    );
                     PlanWithCorrespondingSort {
                         plan: new_plan,
                         sort_onwards: vec![Some(new_tree)],
@@ -658,21 +649,19 @@ fn analyze_window_sort_removal(
 }
 
 /// Updates child to remove the unnecessary `CoalescePartitions` below it.
-fn update_child_to_change_coalesce(
+fn update_child_to_remove_coalesce(
     child: &mut Arc<dyn ExecutionPlan>,
     coalesce_onwards: &mut Option<ExecTree>,
-    sort_exec: Option<&SortExec>,
 ) -> Result<()> {
     if let Some(coalesce_onwards) = coalesce_onwards {
-        *child = change_corresponding_coalesce_in_sub_plan(coalesce_onwards, sort_exec)?;
+        *child = remove_corresponding_coalesce_in_sub_plan(coalesce_onwards)?;
     }
     Ok(())
 }
 
 /// Removes the `CoalescePartitions` from the plan in `coalesce_onwards`.
-fn change_corresponding_coalesce_in_sub_plan(
+fn remove_corresponding_coalesce_in_sub_plan(
     coalesce_onwards: &mut ExecTree,
-    sort_exec: Option<&SortExec>,
 ) -> Result<Arc<dyn ExecutionPlan>> {
     Ok(
         if coalesce_onwards
@@ -681,24 +670,12 @@ fn change_corresponding_coalesce_in_sub_plan(
             .is::<CoalescePartitionsExec>()
         {
             // We can safely use the 0th index since we have a `CoalescePartitionsExec`.
-            let coalesce_input = coalesce_onwards.plan.children()[0].clone();
-            if let Some(sort_exec) = sort_exec {
-                let sort_expr = sort_exec.expr();
-                if !ordering_satisfy(
-                    coalesce_input.output_ordering(),
-                    Some(sort_expr),
-                    || coalesce_input.equivalence_properties(),
-                ) {
-                    return add_sort_above_child(&coalesce_input, sort_expr.to_vec());
-                }
-            }
-            coalesce_input
+            coalesce_onwards.plan.children()[0].clone()
         } else {
             let plan = coalesce_onwards.plan.clone();
             let mut children = plan.children();
             for item in &mut coalesce_onwards.children {
-                children[item.idx] =
-                    change_corresponding_coalesce_in_sub_plan(item, sort_exec)?;
+                children[item.idx] = remove_corresponding_coalesce_in_sub_plan(item)?;
             }
             plan.with_new_children(children)?
         },
@@ -779,15 +756,11 @@ fn change_finer_sort_in_sub_plan(
     let plan = &sort_onwards.plan;
     // A `SortExec` is always at the bottom of the tree.
     if is_sort(plan) {
-        let prev_layer = plan.children()[0].clone();
+        let mut prev_layer = plan.children()[0].clone();
         let new_sort_expr = get_sort_exprs(plan)?[0..n_sort_expr].to_vec();
-        let updated_plan = add_sort_above_child(&prev_layer, new_sort_expr)?;
-        *sort_onwards = ExecTree {
-            idx: sort_onwards.idx,
-            children: vec![],
-            plan: updated_plan.clone(),
-        };
-        Ok(updated_plan)
+        add_sort_above(&mut prev_layer, new_sort_expr)?;
+        *sort_onwards = ExecTree::new(prev_layer.clone(), sort_onwards.idx, vec![]);
+        Ok(prev_layer)
     } else {
         let mut children = plan.children();
         for item in &mut sort_onwards.children {
@@ -1080,12 +1053,12 @@ mod tests {
         let physical_plan = sort_exec(vec![sort_expr("nullable_col", &schema)], input);
 
         let expected_input = vec![
-            "SortExec: [nullable_col@0 ASC]",
-            "  SortExec: [non_nullable_col@1 ASC]",
+            "SortExec: expr=[nullable_col@0 ASC]",
+            "  SortExec: expr=[non_nullable_col@1 ASC]",
             "    MemoryExec: partitions=0, partition_sizes=[]",
         ];
         let expected_optimized = vec![
-            "SortExec: [nullable_col@0 ASC]",
+            "SortExec: expr=[nullable_col@0 ASC]",
             "  MemoryExec: partitions=0, partition_sizes=[]",
         ];
         assert_optimized!(expected_input, expected_optimized, physical_plan);
@@ -1134,9 +1107,9 @@ mod tests {
         let expected_input = vec![
             "WindowAggExec: wdw=[count: Ok(Field { name: \"count\", data_type: Int64, nullable: true, dict_id: 0, dict_is_ordered: false, metadata: {} }), frame: WindowFrame { units: Range, start_bound: Preceding(NULL), end_bound: CurrentRow }]",
             "  FilterExec: NOT non_nullable_col@1",
-            "    SortExec: [non_nullable_col@1 ASC NULLS LAST]",
+            "    SortExec: expr=[non_nullable_col@1 ASC NULLS LAST]",
             "      WindowAggExec: wdw=[count: Ok(Field { name: \"count\", data_type: Int64, nullable: true, dict_id: 0, dict_is_ordered: false, metadata: {} }), frame: WindowFrame { units: Range, start_bound: Preceding(NULL), end_bound: CurrentRow }]",
-            "        SortExec: [non_nullable_col@1 DESC]",
+            "        SortExec: expr=[non_nullable_col@1 DESC]",
             "          MemoryExec: partitions=0, partition_sizes=[]",
         ];
 
@@ -1144,7 +1117,7 @@ mod tests {
             "WindowAggExec: wdw=[count: Ok(Field { name: \"count\", data_type: Int64, nullable: true, dict_id: 0, dict_is_ordered: false, metadata: {} }), frame: WindowFrame { units: Range, start_bound: CurrentRow, end_bound: Following(NULL) }]",
             "  FilterExec: NOT non_nullable_col@1",
             "    WindowAggExec: wdw=[count: Ok(Field { name: \"count\", data_type: Int64, nullable: true, dict_id: 0, dict_is_ordered: false, metadata: {} }), frame: WindowFrame { units: Range, start_bound: Preceding(NULL), end_bound: CurrentRow }]",
-            "      SortExec: [non_nullable_col@1 DESC]",
+            "      SortExec: expr=[non_nullable_col@1 DESC]",
             "        MemoryExec: partitions=0, partition_sizes=[]",
         ];
         assert_optimized!(expected_input, expected_optimized, physical_plan);
@@ -1166,7 +1139,7 @@ mod tests {
         ];
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
-            "  SortExec: [nullable_col@0 ASC]",
+            "  SortExec: expr=[nullable_col@0 ASC]",
             "    MemoryExec: partitions=0, partition_sizes=[]",
         ];
         assert_optimized!(expected_input, expected_optimized, physical_plan);
@@ -1186,15 +1159,15 @@ mod tests {
         let physical_plan = sort_preserving_merge_exec(sort_exprs, sort);
         let expected_input = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
-            "  SortExec: [nullable_col@0 ASC]",
+            "  SortExec: expr=[nullable_col@0 ASC]",
             "    SortPreservingMergeExec: [nullable_col@0 ASC]",
-            "      SortExec: [nullable_col@0 ASC]",
+            "      SortExec: expr=[nullable_col@0 ASC]",
             "        MemoryExec: partitions=0, partition_sizes=[]",
         ];
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
             "  SortPreservingMergeExec: [nullable_col@0 ASC]",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      MemoryExec: partitions=0, partition_sizes=[]",
         ];
         assert_optimized!(expected_input, expected_optimized, physical_plan);
@@ -1223,11 +1196,11 @@ mod tests {
         let expected_input = vec![
             "RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=10",
             "  RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=1",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
-            "        SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "        SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "          SortPreservingMergeExec: [non_nullable_col@1 ASC]",
-            "            SortExec: [non_nullable_col@1 ASC]",
+            "            SortExec: expr=[non_nullable_col@1 ASC]",
             "              MemoryExec: partitions=0, partition_sizes=[]",
         ];
 
@@ -1264,10 +1237,10 @@ mod tests {
         let expected_input = vec![
             "AggregateExec: mode=Final, gby=[], aggr=[]",
             "  SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
-            "    SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "      RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=1",
             "        SortPreservingMergeExec: [non_nullable_col@1 ASC]",
-            "          SortExec: [non_nullable_col@1 ASC]",
+            "          SortExec: expr=[non_nullable_col@1 ASC]",
             "            MemoryExec: partitions=0, partition_sizes=[]",
         ];
 
@@ -1308,20 +1281,20 @@ mod tests {
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
             "      GlobalLimitExec: skip=0, fetch=100",
             "        LocalLimitExec: fetch=100",
-            "          SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "          SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "            ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
 
         // We should keep the bottom `SortExec`.
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
-            "  SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "  SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "    RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=2",
             "      UnionExec",
             "        ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
             "        GlobalLimitExec: skip=0, fetch=100",
             "          LocalLimitExec: fetch=100",
-            "            SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "            SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "              ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
         assert_optimized!(expected_input, expected_optimized, physical_plan);
@@ -1340,12 +1313,12 @@ mod tests {
         let physical_plan = sort_preserving_merge_exec(sort_exprs, sort);
         let expected_input = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
-            "  SortExec: [nullable_col@0 ASC]",
+            "  SortExec: expr=[nullable_col@0 ASC]",
             "    MemoryExec: partitions=0, partition_sizes=[]",
         ];
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
-            "  SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "  SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "    MemoryExec: partitions=0, partition_sizes=[]",
         ];
         assert_optimized!(expected_input, expected_optimized, physical_plan);
@@ -1370,7 +1343,7 @@ mod tests {
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
             "  UnionExec",
             "    ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
         // should not add a sort at the output of the union, input plan should not be changed
@@ -1401,7 +1374,7 @@ mod tests {
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
             "  UnionExec",
             "    ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC, non_nullable_col@1 ASC], projection=[nullable_col, non_nullable_col]",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
         // should not add a sort at the output of the union, input plan should not be changed
@@ -1435,13 +1408,13 @@ mod tests {
             "SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "  UnionExec",
             "    ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
-            "    SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
         // should remove unnecessary sorting from below and move it to top
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
-            "  SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "  SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "    UnionExec",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
@@ -1475,20 +1448,20 @@ mod tests {
         let expected_input = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
             "  UnionExec",
-            "    SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
             "    ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
         // should adjust sorting in the first input of the union such that it is not unnecessarily fine
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
             "  UnionExec",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
             "    ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
         assert_optimized!(expected_input, expected_optimized, physical_plan);
@@ -1520,15 +1493,15 @@ mod tests {
         let expected_input = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "  UnionExec",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
             "    ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
-            "  SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "  SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "    UnionExec",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
@@ -1571,17 +1544,17 @@ mod tests {
         let expected_input = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
             "  UnionExec",
-            "    SortExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
-            "    SortExec: [nullable_col@0 ASC,non_nullable_col@1 DESC NULLS LAST]",
+            "    SortExec: expr=[nullable_col@0 ASC,non_nullable_col@1 DESC NULLS LAST]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
             "  UnionExec",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
         assert_optimized!(expected_input, expected_optimized, physical_plan);
@@ -1617,7 +1590,7 @@ mod tests {
         let expected_input = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
             "  UnionExec",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
             "    ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
             "    SortPreservingMergeExec: [nullable_col@0 ASC,non_nullable_col@1 ASC]",
@@ -1629,11 +1602,11 @@ mod tests {
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
             "  UnionExec",
-            "    SortExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
             "    ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
             "    SortPreservingMergeExec: [nullable_col@0 ASC]",
-            "      SortExec: [nullable_col@0 ASC]",
+            "      SortExec: expr=[nullable_col@0 ASC]",
             "        RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=1",
             "          ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
@@ -1674,9 +1647,9 @@ mod tests {
         let expected_input = vec![
             "WindowAggExec: wdw=[count: Ok(Field { name: \"count\", data_type: Int64, nullable: true, dict_id: 0, dict_is_ordered: false, metadata: {} }), frame: WindowFrame { units: Range, start_bound: Preceding(NULL), end_bound: CurrentRow }]",
             "  UnionExec",
-            "    SortExec: [nullable_col@0 DESC NULLS LAST]",
+            "    SortExec: expr=[nullable_col@0 DESC NULLS LAST]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC, non_nullable_col@1 ASC], projection=[nullable_col, non_nullable_col]",
-            "    SortExec: [nullable_col@0 DESC NULLS LAST]",
+            "    SortExec: expr=[nullable_col@0 DESC NULLS LAST]",
             "      ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[nullable_col@0 ASC], projection=[nullable_col, non_nullable_col]",
         ];
         let expected_optimized = vec![
@@ -1710,7 +1683,7 @@ mod tests {
         // we should be able to parallelize Sorting also (given that executors in between don't require)
         // single partition.
         let expected_input = vec![
-            "SortExec: [nullable_col@0 ASC]",
+            "SortExec: expr=[nullable_col@0 ASC]",
             "  FilterExec: NOT non_nullable_col@1",
             "    CoalescePartitionsExec",
             "      RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=1",
@@ -1718,8 +1691,8 @@ mod tests {
         ];
         let expected_optimized = vec![
             "SortPreservingMergeExec: [nullable_col@0 ASC]",
-            "  FilterExec: NOT non_nullable_col@1",
-            "    SortExec: [nullable_col@0 ASC]",
+            "  SortExec: expr=[nullable_col@0 ASC]",
+            "    FilterExec: NOT non_nullable_col@1",
             "      RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=1",
             "        ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[nullable_col, non_nullable_col]",
         ];
@@ -1776,6 +1749,47 @@ mod tests {
         Ok(())
     }
 
+    #[tokio::test]
+    async fn test_coalesce_propagate() -> Result<()> {
+        let schema = create_test_schema()?;
+        let source = memory_exec(&schema);
+        let repartition = repartition_exec(source);
+        let coalesce_partitions = Arc::new(CoalescePartitionsExec::new(repartition));
+        let repartition = repartition_exec(coalesce_partitions);
+        let sort_exprs = vec![sort_expr("nullable_col", &schema)];
+        // Add local sort
+        let sort = Arc::new(SortExec::new_with_partitioning(
+            sort_exprs.clone(),
+            repartition,
+            true,
+            None,
+        )) as _;
+        let spm = sort_preserving_merge_exec(sort_exprs.clone(), sort);
+        let sort = sort_exec(sort_exprs, spm);
+
+        let physical_plan = sort.clone();
+        // Sort Parallelize rule should end Coalesce + Sort linkage when Sort is Global Sort
+        // Also input plan is not valid as it is. We need to add SortExec before SortPreservingMergeExec.
+        let expected_input = vec![
+            "SortExec: expr=[nullable_col@0 ASC]",
+            "  SortPreservingMergeExec: [nullable_col@0 ASC]",
+            "    SortExec: expr=[nullable_col@0 ASC]",
+            "      RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=1",
+            "        CoalescePartitionsExec",
+            "          RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=0",
+            "            MemoryExec: partitions=0, partition_sizes=[]",
+        ];
+        let expected_optimized = vec![
+            "SortPreservingMergeExec: [nullable_col@0 ASC]",
+            "  SortExec: expr=[nullable_col@0 ASC]",
+            "    RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=10",
+            "      RepartitionExec: partitioning=RoundRobinBatch(10), input_partitions=0",
+            "        MemoryExec: partitions=0, partition_sizes=[]",
+        ];
+        assert_optimized!(expected_input, expected_optimized, physical_plan);
+        Ok(())
+    }
+
     /// make PhysicalSortExpr with default options
     fn sort_expr(name: &str, schema: &Schema) -> PhysicalSortExpr {
         sort_expr_options(name, schema, SortOptions::default())
