@@ -811,22 +811,22 @@ mod tests {
             common::collect(partial_aggregate.execute(0, task_ctx.clone())?).await?;
 
         let expected = vec![
-            "+---+---+-----------------+",
-            "| a | b | COUNT(1)[count] |",
-            "+---+---+-----------------+",
-            "|   | 1 | 2               |",
-            "|   | 2 | 2               |",
-            "|   | 3 | 2               |",
-            "|   | 4 | 2               |",
-            "| 2 |   | 2               |",
-            "| 2 | 1 | 2               |",
-            "| 3 |   | 3               |",
-            "| 3 | 2 | 2               |",
-            "| 3 | 3 | 1               |",
-            "| 4 |   | 3               |",
-            "| 4 | 3 | 1               |",
-            "| 4 | 4 | 2               |",
-            "+---+---+-----------------+",
+            "+---+-----+-----------------+",
+            "| a | b   | COUNT(1)[count] |",
+            "+---+-----+-----------------+",
+            "|   | 1.0 | 2               |",
+            "|   | 2.0 | 2               |",
+            "|   | 3.0 | 2               |",
+            "|   | 4.0 | 2               |",
+            "| 2 |     | 2               |",
+            "| 2 | 1.0 | 2               |",
+            "| 3 |     | 3               |",
+            "| 3 | 2.0 | 2               |",
+            "| 3 | 3.0 | 1               |",
+            "| 4 |     | 3               |",
+            "| 4 | 3.0 | 1               |",
+            "| 4 | 4.0 | 2               |",
+            "+---+-----+-----------------+",
         ];
         assert_batches_sorted_eq!(expected, &result);
 
@@ -858,22 +858,22 @@ mod tests {
         assert_eq!(batch.num_rows(), 12);
 
         let expected = vec![
-            "+---+---+----------+",
-            "| a | b | COUNT(1) |",
-            "+---+---+----------+",
-            "|   | 1 | 2        |",
-            "|   | 2 | 2        |",
-            "|   | 3 | 2        |",
-            "|   | 4 | 2        |",
-            "| 2 |   | 2        |",
-            "| 2 | 1 | 2        |",
-            "| 3 |   | 3        |",
-            "| 3 | 2 | 2        |",
-            "| 3 | 3 | 1        |",
-            "| 4 |   | 3        |",
-            "| 4 | 3 | 1        |",
-            "| 4 | 4 | 2        |",
-            "+---+---+----------+",
+            "+---+-----+----------+",
+            "| a | b   | COUNT(1) |",
+            "+---+-----+----------+",
+            "|   | 1.0 | 2        |",
+            "|   | 2.0 | 2        |",
+            "|   | 3.0 | 2        |",
+            "|   | 4.0 | 2        |",
+            "| 2 |     | 2        |",
+            "| 2 | 1.0 | 2        |",
+            "| 3 |     | 3        |",
+            "| 3 | 2.0 | 2        |",
+            "| 3 | 3.0 | 1        |",
+            "| 4 |     | 3        |",
+            "| 4 | 3.0 | 1        |",
+            "| 4 | 4.0 | 2        |",
+            "+---+-----+----------+",
         ];
 
         assert_batches_sorted_eq!(&expected, &result);
@@ -919,9 +919,9 @@ mod tests {
             "+---+---------------+-------------+",
             "| a | AVG(b)[count] | AVG(b)[sum] |",
             "+---+---------------+-------------+",
-            "| 2 | 2             | 2           |",
-            "| 3 | 3             | 7           |",
-            "| 4 | 3             | 11          |",
+            "| 2 | 2             | 2.0         |",
+            "| 3 | 3             | 7.0         |",
+            "| 4 | 3             | 11.0        |",
             "+---+---------------+-------------+",
         ];
         assert_batches_sorted_eq!(expected, &result);
@@ -956,7 +956,7 @@ mod tests {
             "+---+--------------------+",
             "| a | AVG(b)             |",
             "+---+--------------------+",
-            "| 2 | 1                  |",
+            "| 2 | 1.0                |",
             "| 3 | 2.3333333333333335 |", // 3, (2 + 3 + 2) / 3
             "| 4 | 3.6666666666666665 |", // 4, (3 + 4 + 4) / 3
             "+---+--------------------+",
