@@ -99,11 +99,11 @@ async fn aggregate_timestamps_count() -> Result<()> {
     .await;
 
     let expected = vec![
-        "+----------------+-----------------+-----------------+---------------+",
-        "| COUNT(t.nanos) | COUNT(t.micros) | COUNT(t.millis) | COUNT(t.secs) |",
-        "+----------------+-----------------+-----------------+---------------+",
-        "| 3              | 3               | 3               | 3             |",
-        "+----------------+-----------------+-----------------+---------------+",
+        "+--------------+---------------+---------------+-------------+",
+        "| COUNT(nanos) | COUNT(micros) | COUNT(millis) | COUNT(secs) |",
+        "+--------------+---------------+---------------+-------------+",
+        "| 3            | 3             | 3             | 3           |",
+        "+--------------+---------------+---------------+-------------+",
     ];
     assert_batches_sorted_eq!(expected, &results);
 
@@ -185,11 +185,11 @@ async fn aggregate_times_count() -> Result<()> {
     .await;
 
     let expected = vec![
-        "+----------------+-----------------+-----------------+---------------+",
-        "| COUNT(t.nanos) | COUNT(t.micros) | COUNT(t.millis) | COUNT(t.secs) |",
-        "+----------------+-----------------+-----------------+---------------+",
-        "| 4              | 4               | 4               | 4             |",
-        "+----------------+-----------------+-----------------+---------------+",
+        "+--------------+---------------+---------------+-------------+",
+        "| COUNT(nanos) | COUNT(micros) | COUNT(millis) | COUNT(secs) |",
+        "+--------------+---------------+---------------+-------------+",
+        "| 4            | 4             | 4             | 4           |",
+        "+--------------+---------------+---------------+-------------+",
     ];
     assert_batches_sorted_eq!(expected, &results);
 
