@@ -28,7 +28,7 @@ use chrono::Duration;
 use datafusion::config::ConfigOptions;
 use datafusion::datasource::TableProvider;
 use datafusion::from_slice::FromSlice;
-use datafusion::logical_expr::{Aggregate, LogicalPlan, Projection, TableScan};
+use datafusion::logical_expr::{Aggregate, LogicalPlan, TableScan};
 use datafusion::physical_plan::metrics::MetricValue;
 use datafusion::physical_plan::ExecutionPlan;
 use datafusion::physical_plan::ExecutionPlanVisitor;
@@ -1051,23 +1051,6 @@ async fn register_aggregate_simple_csv(ctx: &SessionContext) -> Result<()> {
     Ok(())
 }
 
-async fn register_aggregate_null_cases_csv(ctx: &SessionContext) -> Result<()> {
-    // It's not possible to use aggregate_test_100, not enought similar values to test grouping on floats
-    let schema = Arc::new(Schema::new(vec![
-        Field::new("c1", DataType::Int64, true),
-        Field::new("c2", DataType::Float64, true),
-        Field::new("c3", DataType::Int64, false),
-    ]));
-
-    ctx.register_csv(
-        "null_cases",
-        "tests/data/null_cases.csv",
-        CsvReadOptions::new().schema(&schema),
-    )
-    .await?;
-    Ok(())
-}
-
 async fn register_aggregate_csv(ctx: &SessionContext) -> Result<()> {
     let testdata = datafusion::test_util::arrow_test_data();
     let schema = test_util::aggr_test_schema();
@@ -1588,18 +1571,15 @@ async fn nyc() -> Result<()> {
     let optimized_plan = dataframe.into_optimized_plan().unwrap();
 
     match &optimized_plan {
-        LogicalPlan::Projection(Projection { input, .. }) => match input.as_ref() {
-            LogicalPlan::Aggregate(Aggregate { input, .. }) => match input.as_ref() {
-                LogicalPlan::TableScan(TableScan {
-                    ref projected_schema,
-                    ..
-                }) => {
-                    assert_eq!(2, projected_schema.fields().len());
-                    assert_eq!(projected_schema.field(0).name(), "passenger_count");
-                    assert_eq!(projected_schema.field(1).name(), "fare_amount");
-                }
-                _ => unreachable!(),
-            },
+        LogicalPlan::Aggregate(Aggregate { input, .. }) => match input.as_ref() {
+            LogicalPlan::TableScan(TableScan {
+                ref projected_schema,
+                ..
+            }) => {
+                assert_eq!(2, projected_schema.fields().len());
+                assert_eq!(projected_schema.field(0).name(), "passenger_count");
+                assert_eq!(projected_schema.field(1).name(), "fare_amount");
+            }
             _ => unreachable!(),
         },
         _ => unreachable!(),
