@@ -278,15 +278,15 @@ async fn select_distinct_simple_2() {
     let actual = execute_to_batches(&ctx, sql).await;
 
     let expected = vec![
-        "+---------+----------------+",
-        "| c1      | c2             |",
-        "+---------+----------------+",
-        "| 0.00001 | 0.000000000001 |",
-        "| 0.00002 | 0.000000000002 |",
-        "| 0.00003 | 0.000000000003 |",
-        "| 0.00004 | 0.000000000004 |",
-        "| 0.00005 | 0.000000000005 |",
-        "+---------+----------------+",
+        "+---------+---------+",
+        "| c1      | c2      |",
+        "+---------+---------+",
+        "| 0.00001 | 1.0e-12 |",
+        "| 0.00002 | 2.0e-12 |",
+        "| 0.00003 | 3.0e-12 |",
+        "| 0.00004 | 4.0e-12 |",
+        "| 0.00005 | 5.0e-12 |",
+        "+---------+---------+",
     ];
     assert_batches_eq!(expected, &actual);
 }
@@ -763,11 +763,11 @@ async fn query_on_string_dictionary() -> Result<()> {
     let sql = "SELECT COUNT(d1) FROM test";
     let actual = execute_to_batches(&ctx, sql).await;
     let expected = vec![
-        "+----------------+",
-        "| COUNT(test.d1) |",
-        "+----------------+",
-        "| 2              |",
-        "+----------------+",
+        "+-----------+",
+        "| COUNT(d1) |",
+        "+-----------+",
+        "| 2         |",
+        "+-----------+",
     ];
     assert_batches_eq!(expected, &actual);
 
