@@ -39,6 +39,6 @@ impl AliasGenerator {
     /// Return a unique alias with the provided prefix
     pub fn next(&self, prefix: &str) -> String {
         let id = self.next_id.fetch_add(1, Ordering::Relaxed);
-        format!("{}_{}", prefix, id)
+        format!("{prefix}_{id}")
     }
 }
